
use AdventureWorks2012

select name, * from sys.sql_modules m
join sys.procedures p
on m.object_id = p.object_id
where uses_native_compilation = 1

-- two index seek, two table update
exec Sales.usp_UpdateSalesOrderShipInfo_native newid, NULL, NULL, NULL, NULL, NULL, NULL, NULL
-- seeks, nested loops, stream aggregate
exec Sales.usp_InsertSalesOrder_inmem newid, NULL, NULL, NULL, NULL, NULL, NULL

-- try to produce a plan that uses merge join

create table T1 (a int not null PRIMARY KEY NONCLUSTERED hash( a ) with( bucket_count = 2048), b int, x char(200)) WITH (MEMORY_OPTIMIZED=ON);
create table T2 (a int not null PRIMARY KEY NONCLUSTERED hash( a ) with( bucket_count = 2048), b int, x char(200)) WITH (MEMORY_OPTIMIZED=ON);

drop table t1
drop table t2

-- multicolumn index, range index
create table T1 (a int not null, b int not null, x char(200), constraint pk_t1 primary key nonclustered(a,b)) WITH (MEMORY_OPTIMIZED=ON);
create table T2 (a int not null PRIMARY KEY NONCLUSTERED(a), b int, x char(200)) WITH (MEMORY_OPTIMIZED=ON);
 
set nocount on
declare @i int
set @i = 0
while @i < 1000
  begin
    insert T1 values (@i * 2, @i * 5, @i)
    insert T2 values (@i * 3, @i * 7, @i)
    set @i = @i + 1
  end

-- merge join without hint
select T1.a, T1.b, T1.x, T2.a, T2.b, T2.x
from dbo.T1 join dbo.T2 on T1.a = T2.a
--option (merge join)

update statistics T1 with fullscan, norecompute
update statistics T2 with fullscan, norecompute
go

CREATE PROCEDURE p1
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN ATOMIC WITH
  (TRANSACTION ISOLATION LEVEL = SNAPSHOT,
   LANGUAGE = N'us_english')
select T1.a, T1.b, T1.x, T2.a, T2.b, T2.x
from dbo.T1 join dbo.T2 on T1.a = T2.a
--option (merge join) -- no query hints
END

CREATE PROCEDURE p2
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN ATOMIC WITH
  (TRANSACTION ISOLATION LEVEL = SNAPSHOT,
   LANGUAGE = N'us_english')
select T1.a, T1.b, T1.x, T2.a, T2.b, T2.x
from dbo.T1 join dbo.T2 on T1.a = T2.a
END

-- nested loops
exec p2

CREATE PROCEDURE p3
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN ATOMIC WITH
  (TRANSACTION ISOLATION LEVEL = SNAPSHOT,
   LANGUAGE = N'us_english')
select T1.a, T1.b, T1.x, T2.a, T2.b, T2.x
from dbo.T1 join dbo.T2 on T1.a = T2.a
order by T2.b
END

-- scan, seek, sort, nested loop
exec p3

CREATE PROCEDURE p4
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN ATOMIC WITH
  (TRANSACTION ISOLATION LEVEL = SNAPSHOT,
   LANGUAGE = N'us_english')
select top(2) T1.a, T1.b, T1.x, T2.a, T2.b, T2.x
from dbo.T1 join dbo.T2 on T1.a = T2.a
order by T2.b
END
go

exec p4
go