
-- Memory used by tables and indexes
-- This is used to produce the Hekaton report

-- Right outer Join with sys.tables to see "system tables"
select t.name, ms.* 
from sys.tables t
join sys.dm_db_xtp_table_memory_stats ms
on t.object_id = ms.object_id

-- Here's the report, a bit more complex
exec sp_executesql @stmt=N'WITH system_allocated_memory(system_allocated_memory_in_mb) AS(
		SELECT ISNULL((SELECT CONVERT(decimal(18,2),(SUM(TMS.memory_allocated_for_table_kb) + SUM(TMS.memory_allocated_for_indexes_kb))/1024.00) 
		FROM [sys].[dm_db_xtp_table_memory_stats] TMS WHERE TMS.object_id <=0), 0.00)),
		table_index_memory(table_used_memory_in_mb, table_unused_memory_in_mb, index_used_memory_in_mb, index_unused_memory_in_mb) AS(
		SELECT ISNULL((SELECT CONVERT(decimal(18,2),(SUM(TMS.memory_used_by_table_kb)/1024.00))), 0.00) AS table_used_memory_in_mb,
			ISNULL((SELECT CONVERT(decimal(18,2),(SUM(TMS.memory_allocated_for_table_kb) - SUM(TMS.memory_used_by_table_kb))/1024.00)), 0.00) AS table_unused_memory_in_mb,
			ISNULL((SELECT CONVERT(decimal(18,2),(SUM(TMS.memory_used_by_indexes_kb)/1024.00))), 0.00) AS index_used_memory_in_mb,
			ISNULL((SELECT CONVERT(decimal(18,2),(SUM(TMS.memory_allocated_for_indexes_kb) - SUM(TMS.memory_used_by_indexes_kb))/1024.00)), 0.00) AS index_unused_memory_in_mb
		FROM [sys].[dm_db_xtp_table_memory_stats] TMS WHERE TMS.object_id > 0)
		SELECT 	s.system_allocated_memory_in_mb, t. table_used_memory_in_mb, t. table_unused_memory_in_mb, t. index_used_memory_in_mb, t. index_unused_memory_in_mb, 
			ISNULL((select top 1 1 from sys.filegroups FG where FG.[type] = ''FX''), 0) as has_memory_optimized_filegroup
		FROM system_allocated_memory s, table_index_memory t',@params=N''
GO

-- Index info: hash_index and nonclustered_index
select * from sys.dm_db_xtp_hash_index_stats          --hash
select * from sys.dm_db_xtp_nonclustered_index_stats  --BWtree
select * from sys.dm_db_xtp_index_stats -- All indexes: Scan, retrieval, phantom information

-- 7 hash indexes
select t.name as table_name, idx.name as index_name, his.* 
from sys.dm_db_xtp_hash_index_stats his
join sys.indexes idx
on his.object_id = idx.object_id and his.index_id = idx.index_id
join sys.tables t
on idx.object_id = t.object_id

-- 5 BWtree 
select t.name as table_name, idx.name as index_name, his.* 
from sys.dm_db_xtp_nonclustered_index_stats his
join sys.indexes idx
on his.object_id = idx.object_id and his.index_id = idx.index_id
join sys.tables t
on idx.object_id = t.object_id

-- 12 total indexes
select t.name as table_name, idx.name as index_name, his.* 
from sys.dm_db_xtp_index_stats his
join sys.indexes idx
on his.object_id = idx.object_id and his.index_id = idx.index_id
join sys.tables t
on idx.object_id = t.object_id

-- 7 tables - insert, update, delete attempts, write conflicts, unique_constraint_violations
select t.name as table_name, his.* 
from sys.dm_db_xtp_object_stats his
join sys.tables t
on his.object_id = t.object_id

-- Memory info: for tables, indexes, and table-variables
select *
from sys.dm_db_xtp_memory_consumers mc  -- object and index info

select t.name as table_name, idx.name as index_name, mc.*
from sys.dm_db_xtp_memory_consumers mc 
join sys.indexes idx
on mc.object_id = idx.object_id and (mc.index_id = idx.index_id or mc.index_id is null)
join sys.tables t
on mc.object_id = t.object_id
order by mc.object_id, mc.index_id




/*
-- No sense in joining these two, memory_stats is summary of memory_consumers
select *
from sys.dm_db_xtp_memory_consumers mc
join sys.dm_db_xtp_table_memory_stats ms
on mc.object_id = ms.object_id

-- no rows in index_stats for data (index_id is null)
select *
from sys.dm_db_xtp_memory_consumers mc
join sys.dm_db_xtp_index_stats ids
on mc.object_id = ids.object_id and mc.index_id = ids.index_id 
*/